package edu.emory.cci.aiw.cvrg.eureka.common.comm;

/*
 * #%L
 * Eureka Common
 * %%
 * Copyright (C) 2012 - 2014 Emory University
 * %%
 * This program is dual licensed under the Apache 2 and GPLv3 licenses.
 * 
 * Apache License, Version 2.0:
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * GNU General Public License version 3:
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import edu.emory.cci.aiw.cvrg.eureka.common.authentication.AuthenticationMethod;
import edu.emory.cci.aiw.cvrg.eureka.common.authentication.LoginType;
import java.util.Date;

/**
 *
 * @author Andrew Post
 */
public final class LocalUser extends User {
	private boolean verified;
	private String verificationCode;
	private String password;
	private Date passwordExpiration;

	public LocalUser() {
		setLoginType(LoginType.INTERNAL);
	}
	
	public boolean isVerified() {
		return verified;
	}

	public void setVerified(boolean inVerified) {
		verified = inVerified;
	}
	
	public String getVerificationCode() {
		return verificationCode;
	}

	public void setVerificationCode(String inVerificationCode) {
		verificationCode = inVerificationCode;
	}
	
	public String getPassword() {
		return password;
	}

	public void setPassword(String inPassword) {
		password = inPassword;
	}

	public Date getPasswordExpiration() {
		return passwordExpiration;
	}

	public void setPasswordExpiration(Date inPasswordExpiration) {
		passwordExpiration = inPasswordExpiration;
	}

	@Override
	public String[] validate() {
		String[] results = super.validate();
		PasswordValidator passwordValidator = new PasswordValidator();
		if (!passwordValidator.validate(this.password)) {
			String[] newResults = new String[results.length + 1];
			System.arraycopy(results, 0, newResults, 0, results.length);
			newResults[results.length] = "Password must be at least 8 characters long.";
			results = newResults;
		}
		return results;
	}
	
	@Override
	public void accept(UserVisitor userVisitor) {
		userVisitor.visit(this);
	}

	@Override
	public AuthenticationMethod authenticationMethod() {
		return AuthenticationMethod.LOCAL;
	}
	
}
