/*
 * #%L
 * Eureka Services
 * %%
 * Copyright (C) 2012 - 2013 Emory University
 * %%
 * This program is dual licensed under the Apache 2 and GPLv3 licenses.
 * 
 * Apache License, Version 2.0:
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * GNU General Public License version 3:
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */
package edu.emory.cci.aiw.cvrg.eureka.services.resource;

import com.google.inject.Inject;
import com.google.inject.persist.Transactional;
import edu.emory.cci.aiw.cvrg.eureka.common.comm.AuthenticationMethod;
import edu.emory.cci.aiw.cvrg.eureka.common.entity.AuthenticationMethodEntity;
import edu.emory.cci.aiw.cvrg.eureka.services.dao.AuthenticationMethodDao;
import java.util.ArrayList;
import java.util.List;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response.Status;
import org.eurekaclinical.standardapis.exception.HttpStatusException;

/**
 *
 * @author Andrew Post
 */
@Transactional
@Path("/protected/authenticationmethods")
@Produces(MediaType.APPLICATION_JSON)
public class AuthenticationMethodResource {

	private final AuthenticationMethodDao authenticationMethodDao;

	@Inject
	public AuthenticationMethodResource(
			AuthenticationMethodDao inAuthenticationMethodDao) {
		this.authenticationMethodDao = inAuthenticationMethodDao;
	}

	@GET
	public List<AuthenticationMethod> getAll() {
		List<AuthenticationMethodEntity> authenticationMethodEntities = this.authenticationMethodDao.getAll();
		List<AuthenticationMethod> authenticationMethods = new ArrayList<>(authenticationMethodEntities.size());
		for (AuthenticationMethodEntity authenticationMethodEntity : authenticationMethodEntities) {
			authenticationMethods.add(authenticationMethodEntity.toAuthenticationMethod());
		}
		return authenticationMethods;
	}

	@GET
	@Path("/{id}")
	public AuthenticationMethod get(@PathParam("id") Long inId) {
		AuthenticationMethodEntity authenticationMethodEntity = this.authenticationMethodDao.retrieve(inId);
		if (authenticationMethodEntity == null) {
			throw new HttpStatusException(Status.NOT_FOUND);
		}
		return authenticationMethodEntity.toAuthenticationMethod();
	}

	@GET
	@Path("/byname/{name}")
	public AuthenticationMethod getByName(@PathParam("name") String inName) {
		AuthenticationMethodEntity authenticationMethodEntity = this.authenticationMethodDao.getByName(edu.emory.cci.aiw.cvrg.eureka.common.authentication.AuthenticationMethod.valueOf(inName));
		if (authenticationMethodEntity == null) {
			throw new HttpStatusException(Status.NOT_FOUND);
		}
		return authenticationMethodEntity.toAuthenticationMethod();
	}
}
